@@ -10,11 +10,12 @@ from .util import *
 from .freesurfer import *
 from .roi import *
 from .subject import *
-from .postproc import *
 from .opt import *
 from .neuron import *
 from .mesh import *
 from .regression import *
+from .util.plot import *
+from .tms_pulse import *
 
 try:
     from .pckg import libeep
@@ -25,7 +26,7 @@ try:
 except ImportError:
     pass
 
-__version__ = "0.2022.9"
+__version__ = "0.2022.12"
 
 # when pipped, datadir is under pynibs
 __testdatadir__ = os.path.join(os.path.dirname(__file__), '..', 'tests', 'data')
