@@ -474,8 +474,8 @@ def get_invalid_coil_parameters(param_dict, coil_position_mean, svd_v, del_obj,
 
     get_invalid_coil_parameters(param_dict, pos_mean, v, del_obj, fn_coil, fn_hdf5_coilpos=None)
 
-    Parameters:
-    -----------
+    Parameters
+    ----------
     param_dict: dict
         Dictionary containing dictionary with 'limits' and 'pdfshape'
         keys: 'x', 'y', 'z', 'psi', 'theta', 'phi'
@@ -490,8 +490,8 @@ def get_invalid_coil_parameters(param_dict, coil_position_mean, svd_v, del_obj,
     fn_hdf5_coilpos: str
         Filename of .hdf5 file to save coil_pos in (incl. path and .hdf5 extension)
 
-    Returns:
-    --------
+    Returns
+    -------
     fail_params: list of int
         Index and combination of failed parameter
     """
@@ -621,9 +621,10 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
 
     Example
     -------
-    Example::
-        pynibs.create_stimsite_hdf5('/data/pt_01756/probands/15484.08/exp/1/experiment_corrected.csv',
-                                     '/data/pt_01756/tmp/test', True, True)
+    .. code-block:: python
+
+        pynibs.create_stimsite_hdf5('/exp/1/experiment_corrected.csv',
+                                    '/stimsite', True, True)
     """
     assert not fn_hdf.endswith('/')
 
@@ -813,12 +814,12 @@ def create_stimsite_from_list(fn_hdf, poslist, datanames=None, data=None, overwr
     Centers and coil orientations are written so disk.
 
     Parameters
-    -----------
-    fn_hdf: basestring
+    ----------
+    fn_hdf: str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
 
-    datanames: basestring or list of basestring
+    datanames: str or list of str
         Dataset names for _data_. Default: None.
 
     data: np.ndarray
@@ -857,12 +858,12 @@ def create_stimsite_from_tmslist(fn_hdf, poslist, datanames=None, data=None, ove
     Centers and coil orientations are written so disk.
 
     Parameters
-    -----------
-    fn_hdf: basestring
+    ----------
+    fn_hdf: str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
 
-    datanames: basestring or list of basestring
+    datanames: str or list of str
         Dataset names for _data_. Default: None.
 
     data: np.ndarray
@@ -901,13 +902,13 @@ def create_stimsite_from_exp_hdf5(fn_exp, fn_hdf, datanames=None, data=None, ove
     This takes an experiment.hdf5 file and creates an .hdf5 + .xdmf tuple for all coil positions for visualization.
 
     Parameters
-    -----------
+    ----------
     fn_exp : str
         Path to experiment.hdf5
-    fn_hdf : basestring
+    fn_hdf : str
         Filename for the resulting .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-    datanames : basestring or list of basestring
+    datanames : str or list of str
         Dataset names for _data_. Default: None.
     data : np.ndarray
         Dataset array with (len(poslist.pos), len(datanames()). Default: None.
@@ -936,18 +937,22 @@ def create_stimsite_from_matsimnibs(fn_hdf, matsimnibs, datanames=None, data=Non
     Centers and coil orientations are written disk.
 
     Parameters
-    -----------
-    fn_hdf: string
+    ----------
+    fn_hdf: str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
     matsimnibs: np.ndarray
         (4, 4, n_pos)
         Matsimnibs matrices containing the coil orientation (x,y,z) and position (p)
-            [ |  |  |  | ]
-            [ x  y  z  p ]
-            [ |  |  |  | ]
-            [ 0  0  0  1 ]
-    datanames: string or list of string
+
+        .. math::
+            \\begin{bmatrix}
+            | & | & | & | \\\\
+            x & y & z & p \\\\
+            | & | & | & | \\\\
+            0 & 0 & 0 & 1 \\\\
+            \\end{bmatrix}
+    datanames: str or list of str
         Dataset names for _data_. Default: None.
     data: np.ndarray, optional
         (len(poslist.pos), len(datanames).
@@ -976,8 +981,8 @@ def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None,
     Centers and coil orientations are written to disk.
 
     Parameters
-    -----------
-    fn_hdf : basestring
+    ----------
+    fn_hdf : str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
     centers : np.ndarray of float [n_pos x 3]
@@ -988,7 +993,7 @@ def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None,
         Coil orientation y-axis (looking at the active (patient) side of the coil pointing up away from the handle)
     m2 : np.ndarray of float [n_pos x 3]
         Coil orientation z-axis (looking at the active (patient) side of the coil pointing to the patient)
-    datanames : basestring or list of basestring [n_data]
+    datanames : str or list of str [n_data]
         Dataset names for _data_. Default: None.
     data : np.ndarray [n_pos, n_data]
         Dataset array with (len(poslist.pos), len(datanames()). Default: None.
@@ -1119,10 +1124,14 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
     if verbose:
         print(f"Loading optimal coil indices from file: {fn_coil_pos_opt}")
 
+    with_intensities = False
     with h5py.File(fn_coil_pos_opt, "r") as f:
         coil_pos_idx = f[root_path + "coil_seq"][:, 0].astype(int)
         goal_fun_val = f[root_path + "coil_seq"][:, 1]
-        intensities_opt = f[root_path + "coil_seq"][:, 2]
+
+        with_intensities = f[root_path + "coil_seq"][:].shape[1] == 3
+        if with_intensities:
+            intensities_opt = f[root_path + "coil_seq"][:, 2]
 
     if verbose:
         print(f"Loading all coil positions from file: {fn_coil_pos}")
@@ -1246,14 +1255,16 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 
     idx_global_sorted = np.zeros(len(goal_fun_val))
     goal_fun_val_sorted = np.zeros(len(goal_fun_val))
-    intensities_opt_sorted = np.zeros(len(intensities_opt))
+    if with_intensities:
+        intensities_opt_sorted = np.zeros(len(intensities_opt))
 
     for i in range(len(idx_global_sorted)):
         for j in range(matsimnibs.shape[2]):
             if (matsimnibs[:, :, j] == matsimnibs_opt_sorted[:, :, i]).all():
                 idx_global_sorted[i] = int(j)
         goal_fun_val_sorted[i] = goal_fun_val[np.where(int(idx_global_sorted[i]) == coil_pos_idx)[0][0]]
-        intensities_opt_sorted[i] = intensities_opt[np.where(int(idx_global_sorted[i]) == coil_pos_idx)[0][0]]
+        if with_intensities:
+            intensities_opt_sorted[i] = intensities_opt[np.where(int(idx_global_sorted[i]) == coil_pos_idx)[0][0]]
 
     # overwrite input file if no output file given
     outpath = fn_coil_pos_opt
@@ -1265,13 +1276,20 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
         print(f"Saving output to: {outpath}")
 
     with h5py.File(outpath, "a") as f:
-        f[root_path + "coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted,
-                                                  intensities_opt_sorted)).transpose()
+        if with_intensities:
+            f[root_path + "coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted,
+                                                      intensities_opt_sorted)).transpose()
+        else:
+            f[root_path + "coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted)).transpose()
+
 
     if print_output:
         fig = plt.figure()
         ax = fig.add_subplot(projection='3d')
-        ax.scatter(coords[routes, 0], coords[routes, 1], coords[routes, 2], s=intensities_opt_sorted * 11, c="k")
+        if with_intensities:
+            ax.scatter(coords[routes, 0], coords[routes, 1], coords[routes, 2], s=intensities_opt_sorted * 11, c="k")
+        else:
+            ax.scatter(coords[routes, 0], coords[routes, 1], coords[routes, 2], c="k")
         ax.plot3D(coords[routes, 0], coords[routes, 1], coords[routes, 2])
         ax.set_xlabel("x in mm")
         ax.set_ylabel("y in mm")
@@ -1284,7 +1302,7 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #     If an invalid position is found, the parameter space is reduced by shrinking the limits.
 #
 #     Parameters:
-#     -----------
+#     ----------
 #     problem : Problem class instance
 #         GPC problem under investigation
 #     coil_position_mean : np.ndarray of float [3 x 4]
