@@ -1,4 +1,5 @@
 import re
+import json
 import pylab
 import pickle
 import warnings
@@ -33,11 +34,12 @@ def sigmoid(x, x0, r, amp):
     """
     Parametrized sigmoid function.
 
-    .. math:: y = \\frac{amp}{1+e^{-r(x-x_0)}}
+    .. math::
+        y = \\frac{amp}{1+e^{-r(x-x_0)}}
 
     Parameters
     ----------
-    x : nparray of float
+    x : np.ndarray of float
         (N_x) X-values the function is evaluated in.
     x0 : float
         Horizontal shift along the abscissa.
@@ -48,8 +50,8 @@ def sigmoid(x, x0, r, amp):
 
     Returns
     -------
-    y : nparray of float [N_x]
-        Function value at argument x
+    y : np.ndarray of float
+        (N_x) Function value at argument x
     """
     y = amp / (1 + np.exp(-r * (x - x0)))
     return y
@@ -59,7 +61,8 @@ def sigmoid4(x, x0, r, amp, y0):
     """
     Parametrized sigmoid function with 4 parameters.
 
-    .. math:: y = y_0 + \\frac{amp - y_0}{1+e^{-r(x-x_0)}}
+    .. math::
+        y = y_0 + \\frac{amp - y_0}{1+e^{-r(x-x_0)}}
 
     Parameters
     ----------
@@ -87,7 +90,8 @@ def sigmoid_log(x, x0, r, amp):
     """
     Parametrized log transformed sigmoid function.
 
-    .. math:: y = \\log\\left(\\frac{amp}{1+e^{-r(x-x_0)}}\\right)
+    .. math::
+        y = \\log\\left(\\frac{amp}{1+e^{-r(x-x_0)}}\\right)
 
     Parameters
     ----------
@@ -113,7 +117,8 @@ def sigmoid4_log(x, x0, r, amp, y0):
     """
     Parametrized log transformed sigmoid function with 4 parameters.
 
-    .. math:: y = \\log\\left(y_0 + \\frac{amp - y_0}{1+e^{-r(x-x_0)}}\\right)
+    .. math::
+        y = \\log\\left(y_0 + \\frac{amp - y_0}{1+e^{-r(x-x_0)}}\\right)
 
     Parameters
     ----------
@@ -138,7 +143,7 @@ def sigmoid4_log(x, x0, r, amp, y0):
 
 def linear(x, m, n):
     """
-    Parametrized linear function
+    Parametrized linear function.
 
     .. math::
         y = mx+n
@@ -179,7 +184,7 @@ def linear_log(x, m, n):
 
     Returns
     -------
-    y : nparray of float
+    y : np.ndarray of float
         (N_x) Function value at x.
     """
     y = m * x + n
@@ -193,7 +198,7 @@ def linear_log(x, m, n):
 
 def exp(x, x0, r, y0):
     """
-    Parametrized exponential function
+    Parametrized exponential function.
 
     .. math::
         y = y_0 + e^{r(x-x_0)}
@@ -211,7 +216,7 @@ def exp(x, x0, r, y0):
 
     Returns
     -------
-    y : nparray of float [N_x]
+    y : np.ndarray of float [N_x]
         Function value at x
     """
     y = y0 + np.exp(r * (x - x0))
@@ -301,56 +306,55 @@ def get_mep_virtual_subject_TVS(x, p1=-5.0818, p2=-2.4677, p3=3.6466, p4=0.42639
                                 sigma2_x_add=2.3671 * 1e-2,
                                 subject_variability=False, trial_variability=True):
     """
-    Creates random mep data using the 3 variability source model from Goetz et al. 2019.
-
-    Goetz, S. M., Alavi, S. M., Deng, Z. D., & Peterchev, A. V. (2019).
-    Statistical Model of Motor-Evoked Potentials.
-    IEEE Transactions on Neural Systems and Rehabilitation Engineering, 27(8), 1539-1545.
-
-    (There are typos in the paper but the code seems to be correct)
-
-    Originally from S. Goetz: https://github.com/sgoetzduke/Statistical-MEP-Model
-
-    rewritten from MATLAB to Python by Konstantin Weise
+    Creates random mep data using the 3 variability source model from [1]_.
+    There are typos in the paper but the code seems to be correct.
+    Originally from S. Goetz: https://github.com/sgoetzduke/Statistical-MEP-Model .
+    Rewritten from MATLAB to Python by Konstantin Weise.
 
     Parameters
     ----------
-    x : ndarray of float [n_x]
-        Normalized stimulator intensities [0 ... 1]
-    p1 : float, optional, default: -5.0818
-        First parameter of sigmoidal hilltype function
-    p2 : float, optional, default: 4.5323
-        Second parameter of sigmoidal hilltype function
-    p3 : float, optional, default: 3.6466
-        Third parameter of sigmoidal hilltype function
-    p4 : float, optional, default: 0.42639
-        Fourth parameter of sigmoidal hilltype function
-    p5 : float, optional, default: 1.6665
-        Fifth parameter of sigmoidal hilltype function
-    mu_y_add : float, optional, default:  10**-5.0818
-        Mean value of additive y variability source
-    mu_y_mult : float, optional, default: -0.9645334
-        Mean value of multiplicative y variability source
-    mu_x_add : float, optional, default: -0.68827324
-        Mean value of additive x variability source
-    sigma_y_add : float, optional, default: 1.4739*1e-6
-        Standard deviation of additive y variability source
-    k : float, optional, default: 0.39316
-        Shape parameter of generalized extreme value distribution
-    sigma2_y_mult : float, optional, default: 2.2759*1e-2
-        Standard deviation of multiplicative y variability source
-    sigma2_x_add : float, optional, default: 2.3671*1e-2
-        Standard deviation of additive x variability source
-    subject_variability : bool, optional, default: False
+    x : np.ndarray of float
+        (n_x) Normalized stimulator intensities [0 ... 1],
+    p1 : float, default: -5.0818
+        First parameter of sigmoidal hilltype function.
+    p2 : float, default: 4.5323
+        Second parameter of sigmoidal hilltype function.
+    p3 : float, default: 3.6466
+        Third parameter of sigmoidal hilltype function.
+    p4 : float, default: 0.42639
+        Fourth parameter of sigmoidal hilltype function.
+    p5 : float, default: 1.6665
+        Fifth parameter of sigmoidal hilltype function.
+    mu_y_add : float, default:  10**-5.0818
+        Mean value of additive y variability source.
+    mu_y_mult : float, default: -0.9645334
+        Mean value of multiplicative y variability source.
+    mu_x_add : float, default: -0.68827324
+        Mean value of additive x variability source.
+    sigma_y_add : float, default: 1.4739*1e-6
+        Standard deviation of additive y variability source.
+    k : float, default: 0.39316
+        Shape parameter of generalized extreme value distribution.
+    sigma2_y_mult : float, default: 2.2759*1e-2
+        Standard deviation of multiplicative y variability source.
+    sigma2_x_add : float, default: 2.3671*1e-2
+        Standard deviation of additive x variability source.
+    subject_variability : bool, default: False
         Choose if shape parameters from IO curve are sampled from random distributions to model subject variability.
         This does not influence the trial-to-trial variability.
-    trial_variability : bool, optional, default: True
+    trial_variability : bool, default: True
         Enable or disable trial-to-trial variability. Disabling it will result in ideal IO curves w/o noise.
 
     Returns
     -------
-    mep : ndarray of float [n_x]
-        Motor evoked potential values in mV
+    mep : np.ndarray of float
+        (n_x) Motor evoked potential values in mV
+
+    References
+    ----------
+    .. [1] Goetz, S. M., Alavi, S. M., Deng, Z. D., & Peterchev, A. V. (2019).
+       Statistical Model of Motor-Evoked Potentials.
+       IEEE Transactions on Neural Systems and Rehabilitation Engineering, 27(8), 1539-1545.
     """
     x = x * 100
     n_x = len(x)
@@ -384,35 +388,38 @@ def get_mep_virtual_subject_TVS(x, p1=-5.0818, p2=-2.4677, p3=3.6466, p4=0.42639
 
 def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0=1e-2, seed=None):
     """
-    Creates random mep data using the 2 variability source model from Goetz et al. 2014 together with a standard 3
+    Creates random mep data using the 2 variability source model from Goetz et al. 2014 [1]_ together with a standard 3
     parametric sigmoid function.
 
-    Goetz, S. M., Luber, B., Lisanby, S. H., & Peterchev, A. V. (2014).
-    A novel model incorporating two variability sources for describing motor evoked potentials.
-    Brain stimulation, 7(4), 541-552.
+    References
+    ----------
+    .. [1] Goetz, S. M., Luber, B., Lisanby, S. H., & Peterchev, A. V. (2014).
+       A novel model incorporating two variability sources for describing motor evoked potentials.
+       Brain stimulation, 7(4), 541-552.
 
     Parameters
     ----------
-    x : ndarray of float [n_x]
-        Normalized stimulator intensities [0 ... 1]
-    x0 : float, optional, default: 0.5
-        Location of turning point sigmoidal function
-    r : float, optional, default: 0.25
-        Steepness of sigmoidal function
-    amp : float, optional, default: 1.0
-        Saturation amplitude of sigmoidal function
-    sigma_x : float, optional, default: 0.1
-        Standard deviation of additive x variability source
-    sigma_y : float, optional, default: 0.1
-        Standard deviation of additive y variability source
-    y0 : float, optional, default: 1e-2
-        y-offset
-    seed: int, optional, default: None
+    x : np.ndarray of float
+        (n_x) Normalized stimulator intensities [0 ... 1].
+    x0 : float, default: 0.5
+        Location of turning point sigmoidal function.
+    r : float, default: 0.25
+        Steepness of sigmoidal function.
+    amp : float, default: 1.0
+        Saturation amplitude of sigmoidal function.
+    sigma_x : float, default: 0.1
+        Standard deviation of additive x variability source.
+    sigma_y : float, default: 0.1
+        Standard deviation of additive y variability source.
+    y0 : float, default: 1e-2
+        y-offset.
+    seed : int, default: None
         Seed to use.
+
     Returns
     -------
-    mep : ndarray of float [n_x]
-        Motor evoked potential values
+    mep : np.ndarray of float
+        (n_x) Motor evoked potential values
     """
     if seed:
         np.random.seed(seed)
@@ -443,46 +450,46 @@ class Mep:
     # TODO: implement list of already fitted function types and their indices in self.fit[] for plot function. \
     #       condition name as attribute
     """
-        Mep object.
+    Mep object.
 
-        Attributes
-        ----------
-        fun : function
-            Kind of function to fit data with (sigmoid / exp / linear)
-        fun_sig : function
-            Best fitting equivalent sigmoidal function (added by add_sigmoidal_bestfit(self))
-        popt : nparray of float [N_para]
-            Fitted optimal function parameters
-        popt_sig : nparray of float [3]
-            Best fitting parameters x0, r, and amp of equivalent sigmoidal function
-        copt : np.array of float [N_para x N_para]
-            covariance matrix of fitted parameters
-        pstd : nparray of float [N_para]
-            Standard deviation of fitted parameters
-        fit : object instance
-            Gmodel object instance of parameter fit
-        x_limits : nparray of float [2]
-            Minimal and maximal value of intensity data
-        y_limits : nparray of float [2]
-            Minimal and maximal value of mep data
-        mt : float
-            Motor threshold (MEP > 50 uV), evaluated from fitted curve, added after fitting
-        """
+    Attributes
+    ----------
+    fun : function
+        Function type to fit data with (``pynibs.sigmoid`` / ``pynibs.exp`` / ``pynibs.linear``).
+    fun_sig : function
+        Best fitting equivalent sigmoidal function (added by ``self.add_sigmoidal_bestfit()``).
+    popt : np.ndarray of float
+        (N_para), Fitted optimal function parameters.
+    popt_sig : np.ndarray of float
+        (3), Best fitting parameters ``x0``, ``r``, and ``amp`` of equivalent sigmoidal function.
+    copt :  np.ndarray of float
+        (N_para, N_para), covariance matrix of fitted parameters.
+    pstd : np.ndarray of float
+        (N_para), Standard deviation of fitted parameters.
+    fit : object instance
+        Gmodel object instance of parameter fit.
+    x_limits : np.ndarray of float
+        (2), Minimal and maximal value of intensity data.
+    y_limits : np.ndarray of float
+        (2), Minimal and maximal value of mep data.
+    mt : float
+        Motor threshold (MEP > 50 uV), evaluated from fitted curve, added after fitting.
+    """
 
     def __init__(self, intensities, mep, intensity_min_threshold=None, mep_min_threshold=None):
         """
-        Initialize Mep object instance
+        Initialize ``Mep`` object.
 
         Parameters
         ----------
-        intensities : nparray of float [N_mep]
-            Intensities of TMS stimulator corresponding to measured MEP amplitudes
-        mep : nparray of float [N_mep]
-            MEP amplitudes in [V] measured during TMS
+        intensities : np.ndarray of float
+            (N_mep), Intensities of TMS stimulator corresponding to measured MEP amplitudes.
+        mep : np.ndarray of float
+            (N_mep), MEP amplitudes in [V] measured during TMS.
         intensity_min_threshold : float
-            Minimum user defined intensity (values below it will be filtered out)
+            Minimum user defined intensity (values below it will be filtered out).
         mep_min_threshold : float
-            Minimum user defined MEP amplitude (values below it will be filtered out)
+            Minimum user defined MEP amplitude (values below it will be filtered out).
         """
 
         self.intensities_orig = intensities
@@ -528,31 +535,26 @@ class Mep:
 
         Parameters
         ----------
-        p0: nparray of float
-            Initial guess of parameter values
+        p0: np.ndarray of float
+            Initial guess of parameter values.
 
         Notes
         -----
         Add Attributes:
 
-        Mep.popt : nparray of float [N_para]
-            Fitted optimal function parameters
-        Mep.copt : nparray of float [N_para x N_para]
-            Covariance matrix of fitted parameters
-        Mep.pstd : nparray of float [N_para]
-            Standard deviation of fitted parameters
+        Mep.popt : np.ndarray of float
+            (N_para) Fitted optimal function parameters.
+        Mep.copt : np.ndarray of float
+            (N_para, N_para) Covariance matrix of fitted parameters.
+        Mep.pstd : np.ndarray of float
+            (N_para) Standard deviation of fitted parameters.
         Mep.fun : function
-            Function mep data was fitted with
+            Function mep data was fitted with.
         Mep.fit : object instance
-            Gmodel object instance of parameter fit
+            Gmodel object instance of parameter fit.
         Mep.mt : float
             Motor threshold (MEP > 50 uV).
         """
-        # filter outliers (disabled)
-        # omask = outliers_mask(mep, m=100.)
-        # intensities = intensities[omask] #+ np.random.randn(intensities.size)*1e-6
-        # mep = mep[omask]
-
         if p0 is None:
             p0 = []
 
@@ -625,38 +627,33 @@ class Mep:
 
         Parameters
         ----------
-        p0 : np.array of float
-            Initial guess of parameter values
+        p0 :  np.ndarray of float
+            Initial guess of parameter values.
         constraints : dict
-            Dictionary with parameter names as keys and [min, max] values as constraints
+            Dictionary with parameter names as keys and [min, max] values as constraints.
         fun : list of functions
-            Functions to incorporate in the multistart fit (e.g. [sigmoid, exp0, linear])
-
+            Functions to incorporate in the multistart fit (e.g.
+            [``pynibs.sigmoid``, ``pynibs.exp0``, ``pynibs.linear``]).
 
         Notes
         -----
         Add Attributes:
 
-        Mep.popt : nparray of float [N_para]
-            Best fitted optimal function parameters
-        Mep.copt : nparray of float [N_para x N_para]
-            Covariance matrix of best fitted parameters
-        Mep.pstd : nparray of float [N_para]
-            Standard deviation of best fitted parameters
+        Mep.popt : np.ndarray of float
+            (N_para) Best fitted optimal function parameters.
+        Mep.copt : np.ndarray of float
+            (N_para, N_para) Covariance matrix of best fitted parameters.
+        Mep.pstd : np.ndarray of float
+            (N_para) Standard deviation of best fitted parameters.
         Mep.fun : function
-            Function of best fit mep data was fitted with
+            Function of best fit mep data was fitted with.
         Mep.fit : list of fit object instances
-            Gmodel object instances of parameter fits
+            Gmodel object instances of parameter fits.
         Mep.best_fit_idx : int
-            Index of best function fit (fit[best_fit_idx])
+            Index of best function fit (``fit[best_fit_idx]``).
         Mep.constraints : dict
-            Dictionary with parameter names as keys and [min, max] values as constraints
+            Dictionary with parameter names as keys and [min, max] values as constraints.
         """
-        # filter outliers (disabled)
-        # omask = outliers_mask(mep, m=100.)
-        # intensities = intensities[omask] #+ np.random.randn(intensities.size)*1e-6
-        # mep = mep[omask]
-
         if fun is None:
             fun = [sigmoid, exp0, linear]
         if p0 is None:
@@ -730,31 +727,31 @@ class Mep:
 
     def run_fit_multistart(self, fun, x, y, p0, constraints=None, verbose=False, n_multistart=100):
         """
-        Run multistart approach to fit data to function. n_multistart optimizations are performed based on random
-        variations of the initial guess parameters p0. The fit with the lowest aic (Akaike information criterion),
+        Run multistart approach to fit data to function. ``n_multistart`` optimizations are performed based on random
+        variations of the initial guess parameters ``p0``. The fit with the lowest AIC (Akaike information criterion),
         i.e. best fit is returned as gmodel fit instance.
 
         Parameters
         ----------
         fun : function
             Function mep data has to be fitted with
-        x : nparray of float [N_data]
-            Independent variable the data is fitted on
-        y : nparray of float [N_data]
-            Dependent data the curve is fitted through
-        p0 : nparray of float
-            Initial guess of parameter values
+        x : np.ndarray of float
+            (N_data), Independent variable the data is fitted on.
+        y : np.ndarray of float
+            (N_data), Dependent data the curve is fitted through.
+        p0 : np.ndarray of float or list of float
+            Initial guess of parameter values.
         constraints : dict, optional
-            Dictionary with parameter names as keys and [min, max] values as constraints
-        verbose : bool
-            Show output messages (True / False (Default))
+            Dictionary with parameter names as keys and [min, max] values as constraints.
+        verbose : bool, default: False
+            Show output messages.
         n_multistart : int
-            Number of repeated optimizations with different starting points to perform
+            Number of repeated optimizations with different starting points to perform.
 
         Returns
         -------
         fit : object instance
-            Gmodel object instance of best parameter fit with lowest parameter variance
+            Gmodel object instance of best parameter fit with lowest parameter variance.
         """
 
         argnames = fun.__code__.co_varnames[1:fun.__code__.co_argcount]
@@ -825,12 +822,12 @@ class Mep:
 
     def calc_motor_threshold(self, threshold):
         """
-        Determine motor threshold of stimulator depending on MEP threshold given in [mV]
+        Determine motor threshold of stimulator depending on MEP threshold given in [mV].
 
         Parameters
         ----------
         threshold : float
-            Threshold of MEP amplitude in [mV]
+            Threshold of MEP amplitude in [mV].
 
         Notes
         -----
@@ -854,37 +851,37 @@ class Mep:
              fontsize_axis=10, fontsize_legend=10, fontsize_label=10, fontsize_title=10, fun=None):
         """
         Plotting mep data and fitted curve together with uncertainties.
-        For (fun is None), the optimal function is plotted.
+        If ``fun == None`, the optimal function is plotted.
 
         Parameters
         ----------
         label : str
-            Title of plot
+            Plot title.
         sigma : float
-            Factor of standard deviations the uncertainty of the fit is plotted with
+            Factor of standard deviations the uncertainty of the fit is plotted with.
         plot_samples : boolean
             Plot sampling curves of the fit in the uncertainty interval.
         show_plot : boolean
-            Show or hide plot window (TRUE / FALSE)
+            Show or hide plot window (``TRUE`` / ``FALSE``).
         fname_plot : str
-            Filename of plot showing fitted data (with .png extension!)
+            Filename of plot showing fitted data (with .png extension).
         ylim : list of float [2]
-            Min and max values of y-axis
+            Min and max values of y-axis.
         fontsize_axis : int
-            Fontsize of axis numbers
+            Fontsize of axis numbers.
         fontsize_legend : int
-            Fontsize of Legend labels
+            Fontsize of Legend labels.
         fontsize_label : int
-            Fontsize of axis labels
+            Fontsize of axis labels.
         fontsize_title : int
-            Fontsize of title
-        fun : str or None (None, 'sigmoid', 'exp', 'linear')
-            Which function to plot
+            Fontsize of title.
+        fun : str or None (None, ``sigmoid``, ``exp``, ``linear``)
+            Which function to plot.
 
         Returns
         -------
         <File> : .png file
-            Plot of Mep data and fit (format: png)
+            Plot of Mep data and fit (format: png).
         """
         p = []
         if show_plot or fname_plot:
@@ -948,7 +945,7 @@ class Mep:
             pylab.legend(loc=2, fontsize=fontsize_legend)
             pylab.xticks(size=fontsize_axis)
             pylab.yticks(size=fontsize_axis)
-            pylab.xlabel('Stimulator intensity [A/$\mu$s]', size=fontsize_label)
+            pylab.xlabel(r'Stimulator intensity [A/$\mu$s]', size=fontsize_label)
 
             if ylabel is not None:
                 pylab.ylabel(ylabel, size=fontsize_label)
@@ -967,17 +964,17 @@ class Mep:
 
         Parameters
         ----------
-        x : nparray of float [N_x]
-            Function values where uncertainty is evaluated
+        x : np.ndarray of float
+            (N_x) Function values where uncertainty is evaluated.
         sigma : float
-            Standard deviation of parameters taken into account when evaluating uncertainty interval
+            Standard deviation of parameters taken into account when evaluating uncertainty interval.
 
         Returns
         -------
-        y_min : nparray of float [N_x]
-            Lower bounds of y-values
-        y_max : nparray of float [N_x]
-            Upper bounds of y-values
+        y_min : np.ndarray of float
+            (N_x) Lower bounds of y-values.
+        y_max : np.ndarray of float
+            (N_x) Upper bounds of y-values.
         """
 
         if not self.fit:
@@ -1004,13 +1001,13 @@ class Mep:
 
         Parameters
         ----------
-        x : nparray of float [N_x]
-            Function arguments
+        x : np.ndarray of float
+            (N_x) Function arguments.
 
         Returns
         -------
-        y : nparray of float [N_x]
-            Function values
+        y : np.ndarray of float
+            (N_x) Function values.
         """
 
         y = self.fun(x, *self.popt)
@@ -1022,136 +1019,158 @@ class Mep:
 
         Parameters
         ----------
-        x: nparray of float [N_x]
-            Function arguments
+        x: np.ndarray of float
+            (N_x) Function arguments.
         p: tuple of float
-            Function parameters
+            Function parameters.
 
         Returns
         -------
-        y: nparray of float [N_x]
-            Function values
+        y: np.ndarray of float
+            (N_x) Function values.
         """
 
         y = self.fun(x, *p)
         return y
 
-        # if multiple p are provided
-        # p = np.array(p)
-        # x = np.array(x).flatten()
-
-        # if p.ndim == 1:
-        #    p = p[np.newaxis, :]
-
-        # n_p_sets = p.shape[0]
-        # n_x = x.shape[0]
-
-        # y = np.zeros((n_p_sets, n_x))
-
-        # for i in range(n_p_sets):
-        #    y[i, :] = self.fun(x, *p[i, :])
-
-        # return y
-
     def eval_fun_sig(self, x, p):
         """
         Evaluating optimally fitted sigmoidal function with optimal parameters in points x.
 
         Parameters
         ----------
-        x: nparray of float [N_x]
-            Function arguments
+        x: np.ndarray of float
+            (N_x) Function arguments.
         p: tuple of float
             Function parameters
 
         Returns
         -------
-        y: nparray of float [N_x]
-            Function values
+        y: np.ndarray of float
+            (N_x) Function values.
         """
 
         y = self.fun_sig(x, *p)
         return y
 
 
-def save_mepobj(obj, fname):
-    """
-    Saving an Mep object in pickle format.
+def read_biosig_emg_data(fn_data, include_first_trigger=False, type="cfs"):
+    try:
+        import biosig
+    except ImportError:
+        ImportError("Please install biosig from pynibs/pkg/biosig folder!")
+        return
 
+    if type == "cfs":  # TODO: also move the TXT reader here?
+        cfs_fn = fn_data
+        cfs_header = json.loads(biosig.header(cfs_fn))
+        cfs_emg = biosig.data(cfs_fn)
 
-    Parameters
-    ----------
-    obj : object
-        Mep object to save
-    fname : str
-        Filename with .pkl extension
+        num_sweeps = cfs_header["NumberOfSweeps"]
+        num_channels = cfs_emg.shape[1]
 
-    Returns
-    --------
-    <File> : .pkl file
-        Mep object in pickle format (.pkl)
-    """
+        total_num_samples = cfs_header["NumberOfSamples"]
+        samples_per_sweep = int(total_num_samples / num_sweeps)
+        sampling_rate = int(cfs_header["Samplingrate"])
 
-    with open(fname, 'wb') as output:
-        pickle.dump(obj, output, -1)
+        # get timestamps
+        tms_pulse_timedelta = datetime.timedelta()
+        # get hour, minute and second
+        time_mep_list = []
+        time_diff_list = []
+        trigger_event_idcs = []
+        if include_first_trigger:
+            trigger_event_idcs.append(0)
+            time_diff_list.append(0)
+            time_mep_list.append(
+                datetime.datetime.strptime(
+                    cfs_header["EVENT"][0]["TimeStamp"], '%Y-%b-%d %H:%M:%S'
+                )
+                -
+                datetime.timedelta(
+                    seconds=float(cfs_header["EVENT"][0]["POS"])
+                )
+            )
 
+        # convert time string into integer
+        for event in cfs_header["EVENT"]:
+            date = datetime.datetime.strptime(event["TimeStamp"], '%Y-%b-%d %H:%M:%S')
 
-def load_mepobj(fname):
-    """
-    Loading Mep object.
+            # we are interested in the tms pulse time, so add it to ts
+            date += tms_pulse_timedelta
+            time_mep_list.append(date)
+            time_diff_list.append((date - time_mep_list[0]).total_seconds())
 
+            # compute indices in data block corresponding to the events
+            if event["TYP"] == "0x7ffe":
+                trigger_event_idcs.append(
+                    round(event["POS"] * cfs_header["Samplingrate"])
+                )
 
-    Parameters
-    ----------
-    fname : str
-        Filename of Mep object with .pkl extension
+        num_sweeps = min(num_sweeps, len(trigger_event_idcs))
 
-    Returns
-    --------
-    object : pyfempp.Mep
-    """
+        emg_data = np.zeros((num_sweeps, num_channels, samples_per_sweep), dtype=np.float32)
 
-    with open(fname, 'rb') as f:
-        return pickle.load(f, encoding='bytes')
+        for c_idx in range(num_channels):
+            # Use emg data startinng from the index of the first trigger event
+            # assumptions:
+            # - after an initial offset all emg data were captured consecutively
+            # - the first emg data frame may be captured without an explicit TMS
+            #   tigger (eg. by checking the "write to disk" option)
+            # - if we had dropouts in between the emg data block (not just at the
+            #   beginning) we would need to adhere to the entire trigger_event_indices
+            #   list.
+            emg_data[:, c_idx, :] = np.reshape(
+                cfs_emg[trigger_event_idcs[0]:, c_idx],
+                (num_sweeps, samples_per_sweep)
+            )
+
+        return emg_data, time_diff_list, num_sweeps, num_channels, samples_per_sweep, sampling_rate
 
 
 def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=0, channels=None, time_format="delta",
-                     plot=False):
+                     plot=False, start_mep=18, end_mep=35):
     """
     Read EMG data from CED .cfs or .txt file and returns MEP amplitudes.
 
     Parameters
     ----------
     mep_fn : string
-        path to ...cfs-file or .txt file (Signal export)
+        path to .cfs-file or .txt file (Signal export).
     tms_pulse_time : float
-        Time in [s] of TMS pulse as specified in signal
-    drop_mep_idx : List of int or None (Default: None)
+        Time in [s] of TMS pulse as specified in signal.
+    drop_mep_idx : List of int or None, default: None
         Which MEPs to remove before matching.
     cfs_data_column : int or list of int
-        Column(s) of dataset in cfs file. +1 for .txt
-    channels : list of str, optional, default: None
-        Channel names
-    time_format : str, optional, default="delta"
-        Format of mep time stamps in time_mep_lst to return
-        - "delta": returns list of datetime.timedelta in seconds
-        - "hms": returns datetime.datetime(year, month, day, hour, minute, second, microsecond)
-    plot : bool, optional, default: False
+        Column(s) of dataset in cfs file. +1 for .txt.
+    channels : list of str, default: None
+        Channel names.
+    time_format : str, default:"delta"
+        Format of mep time stamps in time_mep_lst to return.
+
+        * ``"delta"`` returns list of datetime.timedelta in seconds
+        * ``"hms"`` returns datetime.datetime(year, month, day, hour, minute, second, microsecond)
+    plot : bool, default: False
+        Plot MEPs.
+    start_mep : float, default: 18
+        Start of time frame after TMS pulse where p2p value is evaluated (in ms).
+    end_mep : float, default: 35
+        End of time frame after TMS pulse where p2p value is evaluated (in ms).
 
     Returns
     -------
-    p2p_array : np.ndarray of float [N_stim]
-        Peak to peak values of N sweeps
+    p2p_array : np.ndarray of float
+        (N_stim) Peak to peak values of N sweeps.
     time_mep_lst : list of datetime.timedelta
         MEP-timestamps
-    mep_raw_data : np.ndarray of float [N_channel x N_stim x N_samples]
-        Raw (unfiltered) MEP data
-    mep_filt_data : np.ndarray of float [N_channel x N_stim x N_samples]
-        Filtered MEP data (Butterworth lowpass filter)
-    time : np.ndarray of float [N_samples]
-        Time axis corresponding to MEP data
-    mep_onset_array : np.ndarray of float [S_samples]
-        MEP onset after TMS pulse
+    mep_raw_data : np.ndarray of float
+        (N_channel, N_stim, N_samples) Raw (unfiltered) MEP data.
+    mep_filt_data : np.ndarray of float
+        (N_channel, N_stim, N_samples) Filtered MEP data (Butterworth lowpass filter).
+    time : np.ndarray of float
+        (N_samples) Time axis corresponding to MEP data.
+    mep_onset_array : np.ndarray of float
+        (S_samples) MEP onset after TMS pulse.
     """
     # convert pulse time to datetime object in case of "delta"
     if time_format == "delta":
@@ -1310,7 +1329,9 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
             tmp[i_channel, i_zap, 2] = calc_p2p(sweep=mep_raw_arr[i_channel, i_zap, :],
                                                 tms_pulse_time=tms_pulse_time,
                                                 sampling_rate=sampling_rate,
-                                                fn_plot=None)
+                                                fn_plot=None,
+                                                start_mep=start_mep,
+                                                end_mep=end_mep)
 
     p2p_arr = np.zeros((tmp.shape[0], tmp.shape[1]))
     mep_onset_arr = np.zeros((tmp.shape[0], tmp.shape[1]))
@@ -1347,23 +1368,18 @@ def calc_p2p_old_exp0(sweep, start_mep=None, end_mep=None, tms_pulse_time=None,
     Calc peak-to-peak values of an mep sweep.
     This version was probably used in the ancient times of experiment 0.
 
-
     Parameters
     ----------
-    sweep : np.array of float [Nx1]
-        Input curve
-
-    start_mep: None
+    sweep :  np.ndarray of float
+        (Nx1) Input curve.
+    start_mep : None
         Not used.
-
-     end_mep: None
+    end_mep : None
         Not used.
-
     tms_pulse_time : None
         Not used.
-
     sampling_rate : int (Default: 2000)
-        Sampling rate in Hz
+        Sampling rate in Hz.
 
     Returns
     -------
@@ -1429,7 +1445,7 @@ def calc_p2p_old_exp1(sweep, start_mep=18, end_mep=35, tms_pulse_time=None, samp
 
     Parameters
     ----------
-    sweep : np.array of float [Nx1]
+    sweep :  np.ndarray of float [Nx1]
         Input curve
 
      start_mep: Int or Float (Default: 18)
@@ -1487,7 +1503,7 @@ def calc_p2p(sweep, tms_pulse_time=.2, start_mep=20, end_mep=35, measurement_sta
 
     Parameters
     ----------
-    sweep : np.array of float [Nx1]
+    sweep :  np.ndarray of float [Nx1]
         Input curve
     tms_pulse_time : float (Default: .2)
         onset time of TMS pulse trigger in [s]
@@ -1501,14 +1517,14 @@ def calc_p2p(sweep, tms_pulse_time=.2, start_mep=20, end_mep=35, measurement_sta
         Sampling rate in Hz
     cutoff_freq : int (Default: 500)
         Desired cutoff frequency of the filter, Hz
-    fn_plot : str, optional, default: None
+    fn_plot : str, default: None
         Filename of sweep to plot (.png). If None, plot is omitted.
 
     Returns
     -------
     p2p : float
         Peak-to-peak value of input curve
-    sweep_filt : np.array of float
+    sweep_filt :  np.ndarray of float
         Filtered input curve (Butter lowpass filter with specified cutoff_freq)
     onset : float
         MEP onset after tms_pulse_time
@@ -1553,10 +1569,11 @@ def calc_p2p(sweep, tms_pulse_time=.2, start_mep=20, end_mep=35, measurement_sta
     onset_max -= tms_pulse_time
 
     if fn_plot is not None:
-        timepoints = [idx_to_time_conversion(i) for i in np.arange(len(sweep))]
+        timepoints = np.asarray([idx_to_time_conversion(i) for i in np.arange(len(sweep))], dtype=np.float32)
         sweep_min_idx = np.argmin(sweep_filt[srch_win_start:srch_win_end]) + srch_win_start
         sweep_max_idx = np.argmax(sweep_filt[srch_win_start:srch_win_end]) + srch_win_start
 
+        timepoints /= 1000
         plt.figure(figsize=[4.07, 3.52])
         plt.plot(timepoints, sweep)
         plt.plot(timepoints, sweep_filt)
@@ -1567,9 +1584,8 @@ def calc_p2p(sweep, tms_pulse_time=.2, start_mep=20, end_mep=35, measurement_sta
         plt.grid()
         plt.legend(["raw", "filtered", "p2p"], loc='upper right')
 
-        # plt.xlim([np.max((tms_pulse_time-0.01, np.min(t))), np.min((t[tms_pulse_sample+end_mep]+0.1, np.max(t)))])
         plt.xlim([np.max((tms_pulse_time - 0.01, np.min(timepoints))),
-                  np.min((timepoints[srch_win_end] + 0.1, np.max(timepoints)))])
+                  np.min((timepoints[srch_win_end] + .1, np.max(timepoints)))])
         plt.ylim([-1.1 * np.abs(sweep_min), 1.1 * np.abs(sweep_max)])
 
         plt.xlabel("time in s", fontsize=11)
@@ -1587,14 +1603,16 @@ def get_mep_sampling_rate(cfs_path):
     Returns sampling rate [Hz] for CED Signal EMG data in .cfs, .mat or .txt file.
 
     The sampling rate is saved in the cfs header like this:
-    ...Samplingrate"\t: 3999.999810,\n...
+    .. code-block:: sh
+       ``Samplingrate"\t: 3999.999810,\n``
 
     Parameters
     ----------
-    cfs_path : string with path to ...cfs-file or .txt file
+    cfs_path : str
+        Path to .cfs file or .txt file.
 
     Returns
-    --------
+    -------
     float : sampling rate
     """
     if cfs_path.endswith('.cfs'):
@@ -1610,7 +1628,6 @@ def get_mep_sampling_rate(cfs_path):
         sr_start = idx_a.end() + 4  # 'Samplingrate"\t: '
         return float(cfs_header[sr_start:idx_b.start() + idx_a.end()])
 
-
     elif cfs_path.endswith('.mat'):
         mep_data = spio.loadmat(cfs_path, struct_as_record=False, squeeze_me=True)
         for k in mep_data.keys():
@@ -1649,16 +1666,16 @@ def butter_lowpass(cutoff, fs, order=5):
     Parameters
     ----------
     cutoff : float
-        Cutoff frequency in [Hz]
+        Cutoff frequency in [Hz].
     fs : float
-        Sampling frequency in [Hz]
-    order : int
-        Filter order
+        Sampling frequency in [Hz].
+    order : int, default: 5
+        Filter order.
 
     Returns
     -------
-    b, a : ndarray, ndarray
-        Numerator (b) and denominator (a) polynomials of the IIR filter
+    b, a : np.ndarray, np.ndarray
+        Numerator (b) and denominator (a) polynomials of the IIR filter.
     """
 
     nyq = 0.5 * fs
@@ -1669,9 +1686,11 @@ def butter_lowpass(cutoff, fs, order=5):
 
 def butter_lowpass_filter(data, cutoff, fs, order=5):
     """
+    Applies Butterworth lowpass filter
+
     Parameters
     ----------
-    data : nparray of float [N_samples]
+    data : np.ndarray of float [N_samples]
         Input of the digital filter
     cutoff : float
         Cutoff frequency in [Hz]
@@ -1682,7 +1701,7 @@ def butter_lowpass_filter(data, cutoff, fs, order=5):
 
     Returns
     -------
-    y : nparray [N_samples]
+    y : np.ndarray [N_samples]
         Output of the digital filter
     """
 
@@ -1698,12 +1717,12 @@ def get_time_date(cfs_paths):
     Parameters
     ----------
     cfs_paths : str
-        Path to .cfs mep file
+        Path to .cfs mep file.
 
     Returns
     -------
     time_date : str
-        Date an time
+        Date an time.
     """
 
     cfs_header = biosig.header(cfs_paths[0])
