@@ -5,9 +5,55 @@ from collections import OrderedDict
 
 import numpy as np
 import pandas as pd
+import datetime
+import h5py
 
 import pynibs
 
+class BrainsightCSVParser:
+    class State:
+        FILE_HEADER = 0
+        SKIPPING = 1
+        DATA_HEADER = 2
+        DATA = 3
+        FIN = 4
+
+    def __init__(self):
+        self.current_state = BrainsightCSVParser.State.FILE_HEADER
+        self.header_file = ""
+        self.header_data = []
+        self.data = []
+
+    def check_state_transition(self, line):
+        new_state = BrainsightCSVParser.State.SKIPPING
+        if line.startswith("# Sample Name"):  # data header info
+            new_state = BrainsightCSVParser.State.DATA_HEADER
+        elif line.startswith("Sample "):
+            new_state = BrainsightCSVParser.State.DATA
+        elif line.startswith("# Target Name"):
+            new_state = BrainsightCSVParser.State.SKIPPING
+        elif line.startswith("# "):
+            # we are still parsing the file header
+            if self.current_state == BrainsightCSVParser.State.FILE_HEADER:
+                new_state = BrainsightCSVParser.State.FILE_HEADER
+            # if we surpassed the 'FILE_HEADER' state already, a line with
+            # a leading '#' is a comment, which should be skipped
+            else:
+                new_state = BrainsightCSVParser.State.SKIPPING
+        # all other lines should be skipped
+
+        self.current_state = new_state
+
+    def parse_line(self, line):
+        if self.current_state == BrainsightCSVParser.State.FILE_HEADER:
+            self.header_file += line
+        elif self.current_state == BrainsightCSVParser.State.DATA_HEADER:
+            line = line[2:-1]  # strip leading comment character '# ' and trailing newline '\n'
+            self.header_data = line.split('\t')
+        elif self.current_state == BrainsightCSVParser.State.DATA:
+            line = line[:-1]  # strip trailing newline '\n'
+            data_row = line.split('\t')
+            self.data.append(data_row)
 
 def write_targets_brainsight(targets, fn_out, names=None, overwrite=True):
     """
@@ -97,9 +143,117 @@ def read_targets_brainsight(fn):
     return m_nnav
 
 
-def merge_exp_data(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
-                   remove_coil_skin_distance_outlier=True, coil_distance_corr=True,
-                   verbose=False, plot=False):
+def create_merged_nnav_emg_file_brainsight(
+        fn_brainsight_nnav_info,
+        fn_emg_info,
+        fn_out,
+        p2p_window_start = 18,
+        p2p_window_end = 35
+):
+    # read EMG information
+    _, ext = os.path.splitext(fn_emg_info)
+
+    # @TODO: make more generic: pynibs.get_mep_elements
+    if ext == ".mat":
+        with h5py.File(fn_emg_info, 'r') as data_h5:
+            emg_data = data_h5["/tms_loco_120rmt_wave_data/values"][:]
+            emg_start = data_h5["/tms_loco_120rmt_wave_data/start"][0][0]
+            emg_res = data_h5["/tms_loco_120rmt_wave_data/interval"][0][0]
+            if emg_res < .1: # time step between samples might be stored in seconds
+                emg_res *= 1e3
+
+            num_stim = emg_data.shape[0]
+            num_channels = emg_data.shape[1]
+            num_emg_samples = emg_data.shape[2]
+
+            emg_timestamps_h5_ref = data_h5["/tms_loco_120rmt_wave_data/frameinfo/start"][:]
+            emg_timestamps = []
+            for stim_idx in range(num_stim):
+                emg_timestamps.append(data_h5[emg_timestamps_h5_ref[0][stim_idx]][0][0])
+            emg_timestamps = np.asarray(emg_timestamps, dtype=np.float32)
+            emg_timestamps -= emg_timestamps[0]
+    elif ext == ".cfs":
+        emg_data, emg_timestamps, num_stim, num_channels, num_emg_samples, sampling_rate = \
+            pynibs.read_biosig_emg_data(fn_data=fn_emg_info, include_first_trigger=True, type="cfs")
+        emg_res = 1000 / sampling_rate # [ms]
+        emg_start = 0 # EMG start time not included in CSF file / always 0 for CED Signal (?)
+    else:
+        print(f"[Error] create_merged_nnav_emg_file_brainsight: unsupported EMG data file extension: {ext}"
+              "Only CED Signal files (*.cfs) or MATLAB v.7.3 files are supported."
+              "Does the file you specified have a file extension at all?")
+        return 1
+
+    # read nnav coil information
+    with open(fn_brainsight_nnav_info, 'r') as data_txt_f:
+        csv_parser = BrainsightCSVParser()
+        while True:
+            line = data_txt_f.readline()
+            if not line:
+                break
+            csv_parser.check_state_transition(line)
+            csv_parser.parse_line(line)
+
+        nnav_timestamps = []
+        start_time_string = current_time_string = csv_parser.data[0][24] # 25. column of CSv data = time stamp
+        start_time = datetime.datetime.strptime(start_time_string, '%H:%M:%S.%f')
+        for c_idx in range(0,num_stim):
+            current_time_string = csv_parser.data[c_idx][24] # 25. column of CSv data = time stamp
+            current_time = datetime.datetime.strptime(current_time_string, '%H:%M:%S.%f')
+            nnav_timestamps.append((current_time - start_time).total_seconds())
+
+        nnav_timestamps = np.asarray(nnav_timestamps, dtype=np.float32)
+
+    # verify timesteps
+    timestamp_difference = np.abs(nnav_timestamps - emg_timestamps)
+    if np.any(timestamp_difference > 1):
+        print("[ WARNING ] Detected time stamp differences larger than 1 second. "
+              "Please check whether data from the neuronavigation are aligned with the EMG data.")
+
+    # append EMG info to the nnav coil info
+    # 1) header
+    header_out = csv_parser.header_data.copy()
+    header_out.append("EMG Start")
+    header_out.append("EMG End")
+    header_out.append("EMG Res.")
+    header_out.append("EMG Channels")
+    header_out.append("EMG Window Start")
+    header_out.append("EMG Window End")
+    for c_idx in range(num_channels):
+        header_out.append(f"EMG Peak-to-peak {c_idx+1}")
+        header_out.append(f"EMG Data {c_idx+1}")
+
+    # 2) data
+    data_text_out = []
+    for stim_idx in range(len(emg_data)):
+        # take what we already have in the nnav data
+        row = csv_parser.data[stim_idx].copy()
+        # extend it by the EMG data
+        row.extend([str(emg_start), # EMG Start
+                   "(null)", # EMG End
+                   str(emg_res), # EMG Res.
+                   str(num_channels), # EMG Channels
+                   str(p2p_window_start), # EMG Window Start
+                   str(p2p_window_end)]) # EMG Window End
+        for c_idx in range(num_channels):
+            row.append("0") #EMG Peak-to-peak c_idx+1
+            emg_data_list = emg_data[stim_idx,c_idx,:].tolist()
+            row.append( # EMG data
+                ';'.join(map(str,emg_data_list))
+            )
+        data_text_out.append('\t'.join(row))
+
+    with open(fn_out, 'w') as outfile:
+        outfile.write(csv_parser.header_file)
+        outfile.write("# "+'\t'.join(header_out))
+        outfile.write('\n')
+        outfile.write('\n'.join(data_text_out))
+        outfile.write('\n')
+        outfile.write("# Session Name: END")
+
+
+def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
+                              remove_coil_skin_distance_outlier=True, coil_distance_corr=True,
+                              verbose=False, plot=False, start_mep=18, end_mep=35):
     """
     Merge the TMS coil positions and the mep data into an experiment.hdf5 file.
 
@@ -121,6 +275,10 @@ def merge_exp_data(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
         Plot output messages
     plot : bool, optional, default: False
         Plot MEPs and p2p evaluation
+    start_mep : float, optional, default: 18
+        Start of time frame after TMS pulse where p2p value is evaluated (in ms)
+    end_mep : float, optional, default: 35
+        End of time frame after TMS pulse where p2p value is evaluated (in ms)
     """
 
     nii_exp_path_lst = subject.exp[exp_idx]['fn_mri_nii']
@@ -131,19 +289,56 @@ def merge_exp_data(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
     fn_coil = subject.exp[exp_idx]['fn_coil'][0][0]
     fn_mesh_hdf5 = subject.mesh[mesh_idx]['fn_mesh_hdf5']
     temp_dir = os.path.join(os.path.split(subject.exp[exp_idx]['fn_exp_hdf5'][0])[0],
-                            "pynibs.nnav2simnibs",
+                            "nnav2simnibs",
                             f"mesh_{mesh_idx}")
-    fn_data = subject.exp[exp_idx]["fn_data"][0]
+    fn_data = None
+    if isinstance(subject.exp[exp_idx]["fn_data"], list):
+        try:
+            fn_data = subject.exp[exp_idx]["fn_data"][0][0]
+        except IndexError as e:
+            fn_data = subject.exp[exp_idx]["fn_data"][0]
+    elif isinstance(subject.exp[exp_idx]["fn_data"], str):
+        fn_data = subject.exp[exp_idx]["fn_data"]
+    else:
+        print("[Error] Invalid exp[*]['fn_data']: "
+              "either specify the path as a single string or as a nested list of strings [['path']]!"
+              "Will attempt to read EMG data from exp[*]['fn_tms_nav']")
+
+    if isinstance(subject.exp[exp_idx]["fn_tms_nav"], list):
+        try:
+            fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0][0]
+        except IndexError as e:
+            fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0]
+    elif isinstance(subject.exp[exp_idx]["fn_tms_nav"], str):
+        fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0][0]
+    else:
+        print("[Fatal error] Invalid exp[*]['fn_tms_nav']: "
+              "either specify the path as a single string or as a nested list of strings [['path']]!")
+        return 1
+
+    if fn_data is not None:
+        fn_out, out_ext = os.path.splitext(fn_tms_nav)
+        fn_out += "_merged" + out_ext
+
+        if not os.path.exists(fn_out):
+            create_merged_nnav_emg_file_brainsight(
+                fn_brainsight_nnav_info=fn_tms_nav,
+                fn_emg_info=fn_data,
+                fn_out=fn_out
+            )
+
+        fn_tms_nav = fn_out
 
     # read Brainsight data
     ######################
     if verbose:
-        print(f"Reading Brainsight data from file: {fn_data}")
+        print(f"Reading Brainsight data from file: {fn_tms_nav}")
 
     d_bs = OrderedDict()
 
-    with io.open(fn_data, 'r') as f:
+    with io.open(fn_tms_nav, 'r') as f:
         lines = f.readlines()
+
     start_idx = [i + 1 for i, l in enumerate(lines) if l.startswith("# Sample Name")][0]
     stop_idx = [i for i, l in enumerate(lines) if l.startswith("# Session Name")]
 
@@ -169,7 +364,7 @@ def merge_exp_data(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
 
         for i_key, key in enumerate(d_bs.keys()):
             if key in ["Sample Name", "Session Name", "Creation Cause", "Crosshairs Driver", "Date", "Time",
-                       "EMG Channels"]:
+                       "EMG Channels", "Assoc. Target"]:
                 d_bs[key].append(line[i_key])
             else:
                 if ";" in line[i_key]:
@@ -215,17 +410,17 @@ def merge_exp_data(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
     print(nii_conform_path)
 
     m_simnibs = pynibs.nnav2simnibs(fn_exp_nii=nii_exp_path_lst[0][0],
-                             fn_conform_nii=nii_conform_path,
-                             m_nnav=m_nnav,
-                             nnav_system="brainsight",
-                             mesh_approach="headreco",
-                             fiducials=None,
-                             orientation='RAS',
-                             fsl_cmd=None,
-                             target='simnibs',
-                             temp_dir=temp_dir,
-                             rem_tmp=True,
-                             verbose=verbose)
+                                    fn_conform_nii=nii_conform_path,
+                                    m_nnav=m_nnav,
+                                    nnav_system="brainsight",
+                                    mesh_approach="headreco",
+                                    fiducials=None,
+                                    orientation='RAS',
+                                    fsl_cmd=None,
+                                    target='simnibs',
+                                    temp_dir=temp_dir,
+                                    rem_tmp=True,
+                                    verbose=verbose)
 
     # create dictionary containing stimulation and physiological data
     #################################################################
@@ -276,47 +471,49 @@ def merge_exp_data(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
                     d[key].append(d_bs[key][i])
 
     # add physiological raw data
-    start_mep_ms = 18
-    end_mep_ms = 35
-    channels = np.arange(1, int(d["EMG Channels"][0]) + 1)
+    channels = subject.exp[exp_idx]["channels"]
 
     if verbose:
         print(f"Postprocessing MEP data")
         if plot:
             print(" Creating MEP plots ...")
 
-    for chan in channels:
-        d[f"mep_raw_data_time_{chan}"] = []
-        d[f"mep_filt_data_time_{chan}"] = []
-        d[f"mep_raw_data_{chan}"] = []
-        d[f"mep_filt_data_{chan}"] = []
-        d[f"p2p_brainsight_{chan}"] = []
-        d[f"p2p_{chan}"] = []
-        d[f"mep_latency_{chan}"] = []
+    for c_idx, chan_name in enumerate(channels):
+        # brainsight indexes channels 1-based
+        c_idx += 1
+
+        d[f"mep_raw_data_time_{chan_name}"] = []
+        d[f"mep_filt_data_time_{chan_name}"] = []
+        d[f"mep_raw_data_{chan_name}"] = []
+        d[f"mep_filt_data_{chan_name}"] = []
+        d[f"p2p_brainsight_{chan_name}"] = []
+        d[f"p2p_{chan_name}"] = []
+        d[f"mep_latency_{chan_name}"] = []
 
         for i in range(n_stims):
+            fn_plot = None
             if plot:
-                fn_channel = os.path.join(os.path.split(subject.exp[exp_idx]["fn_data"][0])[0], "plots", str(chan))
+                fn_channel = os.path.join(os.path.dirname(fn_tms_nav), "plots", str(chan_name))
                 fn_plot = os.path.join(fn_channel, f"mep_{i:04}")
                 os.makedirs(fn_channel, exist_ok=True)
 
             # filter data and calculate p2p values
-            p2p, mep_filt_data, latency = pynibs.calc_p2p(sweep=d_bs[f"EMG Data {chan}"][i],
+            p2p, mep_filt_data, latency = pynibs.calc_p2p(sweep=d_bs[f"EMG Data {c_idx}"][i],
                                                           tms_pulse_time=d_bs[f"Offset"][i],
                                                           sampling_rate=1000 / d_bs["EMG Res."][i],
-                                                          start_mep=start_mep_ms, end_mep=end_mep_ms,
+                                                          start_mep=start_mep, end_mep=end_mep,
                                                           measurement_start_time=float(d["EMG Start"][i]),
                                                           fn_plot=fn_plot)
 
-            d[f"mep_raw_data_time_{chan}"].append(np.arange(d_bs["EMG Start"][i],
+            d[f"mep_raw_data_time_{chan_name}"].append(np.arange(d_bs["EMG Start"][i],
                                                             d_bs["EMG End"][i], d_bs["EMG Res."][i]))
-            d[f"mep_filt_data_time_{chan}"].append(
+            d[f"mep_filt_data_time_{chan_name}"].append(
                     np.arange(d_bs["EMG Start"][i], d_bs["EMG End"][i], d_bs["EMG Res."][i]))
-            d[f"mep_raw_data_{chan}"].append(d_bs[f"EMG Data {chan}"][i])
-            d[f"mep_filt_data_{chan}"].append(mep_filt_data)
-            d[f"p2p_brainsight_{chan}"].append(d_bs[f"EMG Peak-to-peak {chan}"][i])
-            d[f"p2p_{chan}"].append(p2p)
-            d[f"mep_latency_{chan}"].append(latency)
+            d[f"mep_raw_data_{chan_name}"].append(d_bs[f"EMG Data {c_idx}"][i])
+            d[f"mep_filt_data_{chan_name}"].append(mep_filt_data)
+            d[f"p2p_brainsight_{chan_name}"].append(d_bs[f"EMG Peak-to-peak {c_idx}"][i])
+            d[f"p2p_{chan_name}"].append(p2p)
+            d[f"mep_latency_{chan_name}"].append(latency)
 
     # set filename of experiment.hdf5
     if subject.exp[exp_idx]["fn_exp_hdf5"] is not None or subject.exp[exp_idx]["fn_exp_hdf5"] != []:
