@@ -222,8 +222,8 @@ def arrays_similar(tm_matrix, tm_matrix_post,  # , tm_mean_last,
     Splitting the comparison into angles and position is angebracht, as the absolute tolereance (atol) should be
     different for angles (degree) and position (millimeter) comparisons.
 
-    Parameters:
-    -----------
+    Parameters
+    ----------
     tm_matrix: array-like, shape = (4,4)
         TMS Navigator triggermarker or instrument marker array
 
@@ -539,7 +539,7 @@ def read_triggermarker_localite(fn_xml):
 
 def merge_exp_data_localite(subject, coil_outlier_corr_cond, remove_coil_skin_distance_outlier, coil_distance_corr,
                             exp_idx=0, mesh_idx=0, drop_mep_idx=None, mep_onsets=None, cfs_data_column=None,
-                            channels=None, verbose=False, plot=False):
+                            channels=None, verbose=False, plot=False, start_mep=18, end_mep=35):
     """
     Merge the TMS coil positions (TriggerMarker) and the mep data into an experiment.hdf5 file.
 
@@ -569,6 +569,10 @@ def merge_exp_data_localite(subject, coil_outlier_corr_cond, remove_coil_skin_di
         Plot output messages
     plot : bool, optional, default: False
         Plot MEPs and p2p evaluation
+    start_mep : float, optional, default: 18
+        Start of time frame after TMS pulse where p2p value is evaluated (in ms)
+    end_mep : float, optional, default: 35
+        End of time frame after TMS pulse where p2p value is evaluated (in ms)
     """
     if channels is None:
         channels = ["channel_0"]
@@ -580,9 +584,7 @@ def merge_exp_data_localite(subject, coil_outlier_corr_cond, remove_coil_skin_di
     fn_exp_hdf5 = subject.exp[exp_idx]['fn_exp_hdf5'][0]
     fn_coil = subject.exp[exp_idx]['fn_coil']
     fn_mesh_hdf5 = subject.mesh[mesh_idx]['fn_mesh_hdf5']
-    temp_dir = os.path.join(os.path.split(subject.exp[exp_idx]['fn_exp_hdf5'][0])[0],
-                            "nnav2simnibs",
-                            f"mesh_{mesh_idx}")
+    temp_dir = os.path.join(os.path.split(subject.exp[exp_idx]['fn_exp_hdf5'][0])[0], "nnav2simnibs")
     subject_obj = subject
 
     # allocate dict
@@ -622,7 +624,9 @@ def merge_exp_data_localite(subject, coil_outlier_corr_cond, remove_coil_skin_di
                                                 channels=channels,
                                                 nnav_system=subject_obj.exp[exp_idx]["nnav_system"],
                                                 mesh_approach=subject_obj.mesh[mesh_idx]["approach"],
-                                                plot=plot))
+                                                plot=plot,
+                                                start_mep=start_mep,
+                                                end_mep=end_mep))
 
         if len(len_conds) == 0:
             len_conds.append(len(dict_lst))
