@@ -771,7 +771,7 @@ class TetrahedraLinear:
 
     def data_elements2nodes(self, data):
         """
-        Transforms an data in tetrahedra into the nodes after Zienkiewicz et al. (1992) [1].
+        Transforms an data in tetrahedra into the nodes after Zienkiewicz et al. (1992) [1]_.
         Can only transform volume data, i.e. needs the data in the surrounding tetrahedra to average it to the nodes.
         Will not work well for discontinuous fields (like E, if several tissues are used).
 
@@ -1254,8 +1254,8 @@ class Mesh:
         """
         Initializes attributes for a headreco mesh.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         approach: str
             'headreco'
             'mri2mesh'
@@ -1293,8 +1293,8 @@ class Mesh:
         """
         Write this mesh' attributes to .hdf5 file.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         fn_hdf5 : str
         check_file_exist : bool
             Check if provided filenames exist, warn if not.
@@ -1352,8 +1352,8 @@ class ROI:
         """
         Write this mesh' attributes to .hdf5 file.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         fn_hdf5 : str
         check_file_exist : bool
             Check if provided filenames exist, warn if not.
