@@ -15,8 +15,8 @@ def calc_tet_volume(points, abs=True):
     """
     Calculate tetrahedra volumes.
 
-    Parameter
-    ---------
+    Parameters
+    ----------
     points: np.ndarray
         shape: (n_tets,4,3)
         [[[Ax, Ay, Az],
@@ -63,8 +63,8 @@ def calc_tri_surface(points):
     """
     Calculate triangle surface areas.
 
-    Parameter
-    ---------
+    Parameters
+    ----------
     points : np.ndarray
         (n_triangles,3,3)
 
@@ -759,8 +759,8 @@ def check_islands_for_single_elm(source_elm, connectivity=None, adjacency=None,
 
     if island_crit == 'any':
         # find elements with only one node in common
-        neighbors_visited = {i: False for i in np.where(adjacency[source_elm] == 1)[0]}
-        island_crit = 2
+        neighbors_visited = {i: False for i in np.where(adjacency[source_elm] >= 1)[0]}
+        island_crit = 1
     elif island_crit == 'node':
         # find elements with only one node in common
         neighbors_visited = {i: False for i in np.where(adjacency[source_elm] == 1)[0]}
@@ -774,7 +774,7 @@ def check_islands_for_single_elm(source_elm, connectivity=None, adjacency=None,
 
     # now visit all elements with 2 or more neighboring elements recursivly.
     # everything that's left over between 1 and 2 neighbors is an island
-    neighs_to_check = set(np.where(adjacency[source_elm] > island_crit)[0].tolist())
+    neighs_to_check = set(np.where(adjacency[source_elm] >= island_crit)[0].tolist())
 
     # add the starting element to the list
     neighs_to_check.add(source_elm)
@@ -787,7 +787,7 @@ def check_islands_for_single_elm(source_elm, connectivity=None, adjacency=None,
         # now add all 2-node neighbors for this element
         # for neigh in np.where(adjacency[elm] > 1)[0]:
         sort = (-adjacency[elm]).argsort()
-        for idx in np.arange((np.bincount(adjacency[elm])[(island_crit + 1):]).sum()):
+        for idx in np.arange((np.bincount(adjacency[elm])[(island_crit):]).sum()):
             i = sort[idx]
             if i not in neighbors_visited or not neighbors_visited[i]:
                 # print(i)
@@ -798,7 +798,7 @@ def check_islands_for_single_elm(source_elm, connectivity=None, adjacency=None,
            neighbors_visited
 
 
-def find_islands(connectivity=None, adjacency=None, island_crit='any', verbose=False):
+def find_islands(connectivity=None, adjacency=None, island_crit='any', verbose=False, largest=False):
     """
     This identifies islands in a mesh. An island is a set of elements, that is only connect
     via a single node to another set of elements.
@@ -810,6 +810,12 @@ def find_islands(connectivity=None, adjacency=None, island_crit='any', verbose=F
         3. Continue recursively with all 2-node-neighbors and visit their 2-node-neighbors
         4. See if any 1-node-neighbors have not been visited with this strategy. If so, an island has been found
 
+    .. figure:: ../../doc/images/find_islands.png
+       :scale: 50 %
+       :alt: Island detection
+
+       Islands are groups of elements that are only connected via a single node/edge to another group.
+
     Parameters
     ----------
     connectivity : np.ndarray, optional
@@ -823,6 +829,8 @@ def find_islands(connectivity=None, adjacency=None, island_crit='any', verbose=F
         'any' -> Elements connected via a single node or single edge are defined as an island.
         'node' -> Elements connected via a single _node_ are defined as an island.
         'edge' -> Elements connected via a single _edge_ are defined as an island.
+    largest : book, default: False
+        Only return largest island, speeds up computation quite a bit if only one large, and many small islands exist.
     verbose : bool, optional
         Print some verbosity information. Default: False
     Returns
@@ -846,17 +854,33 @@ def find_islands(connectivity=None, adjacency=None, island_crit='any', verbose=F
     counter_not_visited = np.zeros(adjacency.shape[0])
     counter_visited = np.zeros(adjacency.shape[0])
 
+    size_island = 0
+    visited_elms = set()
     # go through elements and check for islands
     for elm_source in tqdm(range(adjacency.shape[0]), desc="Checking for islands."):
+        if largest and elm_source in visited_elms:
+            continue
         _, n_not_visited, visited = check_islands_for_single_elm(elm_source,
                                                                  adjacency=adjacency,
                                                                  island_crit=island_crit)
 
         # add stats for each element how often it has been visited
         elms_visited = [i for i, v in visited.items() if v]
-        elms_not_visited = [i for i, v in visited.items() if not v]
-        counter_visited[elms_visited] += 1
-        counter_not_visited[elms_not_visited] += 1
+
+        if not largest:
+            counter_visited[elms_visited] += 1
+            elms_not_visited = [i for i, v in visited.items() if not v]
+            counter_not_visited[elms_not_visited] += 1
+        else:
+            visited_elms.update(elms_visited)
+            if len(elms_visited) > size_island:
+                elms_not_visited = [i for i, v in visited.items() if not v]
+                size_island = len(elms_visited)
+                visited_elms.update(elms_visited)
+                counter_visited = np.zeros(adjacency.shape[0])
+                counter_not_visited = np.zeros(adjacency.shape[0])
+                counter_visited[elms_visited] += 1
+                counter_not_visited[elms_not_visited] += 1
 
         # if an island is found add it to results list
         if n_not_visited:
