@@ -20,7 +20,7 @@ class Subject:
     -----
     **Initialization**
 
-    sub = subject(subject_ID, mesh)
+        sub = pynibs.subject(subject_ID, mesh)
 
     **Parameters**
 
@@ -29,7 +29,6 @@ class Subject:
     fn_mesh : str
          .msh or .hdf5 file containing the mesh information
 
-
     **Subject.seg, segmentation information dictionary**
 
     fn_lh_wm : str
@@ -120,6 +119,80 @@ class Subject:
         self.ps = []  # list containing plot settings
         self.roi = {}  # dict containing the roi information
 
+    def __str__(self):
+        """ Overload method to allow print(subject_object)"""
+        ret = f'{"="*64}\n' \
+              f'Subject ID : {self.id}\n' \
+              f'Folder     : {self.subject_folder}\n' \
+              f'Meshes     : {len(self.mesh.items())}\n' \
+              f'Experiments: {len(self.exp.items())}\n'
+        ret +=f'{"="*64}\n\n'
+        ret += '------------------------------MRI------------------------------\n'
+        if not self.mri:
+            ret += 'None\n'
+        else:
+            for idx_mesh, mri in enumerate(self.mri):
+                ret += f"|- MRI[{idx_mesh}]\n"
+                for k, d in mri.items():
+                    if k == list(mri.keys())[-1]:
+                        pref = '└-'
+                    else:
+                        pref =  '|-'
+                    ret += f'   {pref}  {k: >19}: {d}\n'
+
+        ret += '\n------------------------------MESH------------------------------\n'
+        if not self.mesh:
+            ret += 'None\n'
+        else:
+            for idx_mesh, mesh in self.mesh.items():
+                ret += f"|- Mesh name: {idx_mesh}\n"
+
+                # plot roi keys
+                if idx_mesh in self.roi.keys():
+                    for r_idx, roi in self.roi[idx_mesh].items():
+                        key_list_roi = []
+                        ret += f'  |-- ROI: {r_idx}\n'
+                        for k, d in roi.items():
+                            if d is not None and k != 'name' and k != 'mesh_name':
+                                key_list_roi.append(k)
+                        for k in key_list_roi:
+                            if k == key_list_roi[-1]:
+                                pref = '└-'
+                            else:
+                                pref = '|-'
+                            ret += f'    {pref} {k: >19}: {roi[k]}\n'
+
+                # plot mesh keys
+                key_list_mesh = []
+                for k, d in mesh.items():
+                    if d is not None and k != 'name' and k != 'subject_id':
+                        key_list_mesh.append(k)
+                for k in key_list_mesh:
+                    if k == key_list_mesh[-1]:
+                        pref = '└-'
+                    else:
+                        pref =  '|-'
+                    ret += f'  {pref} {k: >19}: {mesh[k]}\n'
+
+                ret +="\n"
+        ret += '\n----------------------------Experiments-------------------------\n'
+        if not self.exp:
+            ret += 'None\n'
+        else:
+            for idx_exp, exp in self.exp.items():
+                ret += f"|- Exp name: {idx_exp}\n"
+                key_list_exp = []
+                for k, d in exp.items():
+                    if d is not None and k != 'name' and k != 'subject_id':
+                        key_list_exp.append(k)
+                for k in key_list_exp:
+                    if k == key_list_exp[-1]:
+                        pref = '└-'
+                    else:
+                        pref = '|-'
+                    ret += f'  {pref} {k: >19}: {exp[k]}\n'
+        return ret
+
     def add_mesh_info(self, mesh_dict):
         """
         Adding filename information of the mesh to the subject object (multiple filenames possible).
@@ -496,7 +569,7 @@ def load_subject_hdf5(fname):
 
     Returns
     -------
-    subject : Subject object
+    subject : pynibs.subject.Subject
         Loaded Subject object
     """
 
@@ -594,6 +667,10 @@ def load_subject(fname, filetype=None):
         endwith('.pkl') | endswith('.hdf5')
     filetype: str
         Explicitely set file version.
+
+    Returns
+    -------
+    subject : pynibs.subject.Subject
     """
 
     # explicit set fname type
@@ -632,7 +709,7 @@ def load_subject_pkl(fname):
 
     Returns
     -------
-    subject : Subject object
+    subject : pynibs.subject.Subject
         Loaded Subject object
     """
 
