@@ -25,9 +25,10 @@ def biphasic_pulse(t, R=0.0338, L=15.5*1e-6, C=193.6*1e-6, alpha=1089.8, f=2900)
     e: ndarray of float [n_t]
         Normalized electric field time course (can be scaled with electric field)
     """
+
     omega = 2 * np.pi * f
     i = 1/(omega*L) * np.exp(-alpha*t) * np.sin(omega*t)
     e = np.gradient(i)
     e = e/np.max(e)
 
-    return e
+    return i, e
