@@ -8,16 +8,20 @@ def geodesic_dist(nodes, tris, source, source_is_node=True):
 
     This is just a wrapper for the gdist package.
 
-    Example:
-    with h5py.File(fn,'r') as f:
-        tris = f['/mesh/elm/triangle_number_list'][:]
-        nodes = f['/mesh/nodes/node_coord'][:]
-    nodes_dist_ged, tris_dist_ged = geodesic_dist(nodes, tris, 3017)
+    Example
+    -------
 
-    pynibs.write_data_hdf5(data_fn,
-                        data=[tris_dist_ged, nodes_dist_ged],
-                        data_names=["tris_dist_ged", "nodes_dist_ged"])
-    pynibs.write_xdmf(data_fn,hdf5_geo_fn=fn)
+    .. code-block:: python
+
+        with h5py.File(fn,'r') as f:
+            tris = f['/mesh/elm/triangle_number_list'][:]
+            nodes = f['/mesh/nodes/node_coord'][:]
+        nodes_dist_ged, tris_dist_ged = geodesic_dist(nodes, tris, 3017)
+
+        pynibs.write_data_hdf5(data_fn,
+                            data=[tris_dist_ged, nodes_dist_ged],
+                            data_names=["tris_dist_ged", "nodes_dist_ged"])
+        pynibs.write_xdmf(data_fn,hdf5_geo_fn=fn)
 
 
     Parameters
@@ -67,16 +71,19 @@ def euclidean_dist(nodes, tris, source, source_is_node=True):
 
     This is just a wrapper for the gdist package.
 
-    Example:
-    with h5py.File(fn,'r') as f:
-        tris = f['/mesh/elm/triangle_number_list'][:]
-        nodes = f['/mesh/nodes/node_coord'][:]
-    nodes_dist_euc, tris_dist_euc = euclidean_dist(nodes, tris, 3017)
+    Example
+    -------
+    .. code-block:: python
 
-    pynibs.write_data_hdf5(data_fn,
-                        data=[tris_dist_euc, nodes_dist_euc],
-                        data_names=["tris_dist_euc", "nodes_dist_euc", "])
-    pynibs.write_xdmf(data_fn,hdf5_geo_fn=fn)
+        with h5py.File(fn,'r') as f:
+            tris = f['/mesh/elm/triangle_number_list'][:]
+            nodes = f['/mesh/nodes/node_coord'][:]
+        nodes_dist_euc, tris_dist_euc = euclidean_dist(nodes, tris, 3017)
+
+        pynibs.write_data_hdf5(data_fn,
+                            data=[tris_dist_euc, nodes_dist_euc],
+                            data_names=["tris_dist_euc", "nodes_dist_euc", "])
+        pynibs.write_xdmf(data_fn,hdf5_geo_fn=fn)
 
     Parameters
     ----------
@@ -108,8 +115,6 @@ def nrmsd(array, array_ref, error_norm="relative", x_axis=False):
     """
     Determine the normalized root-mean-square deviation between input data and reference data.
 
-    normalized_rms = get_normalized_rms(array, array_ref)
-
     Parameters
     ----------
     array: np.ndarray
@@ -125,7 +130,7 @@ def nrmsd(array, array_ref, error_norm="relative", x_axis=False):
 
     Returns
     -------
-    normalized_rms: ndarray of float [array.shape[1]]
+    normalized_rms: np.ndarray of float [array.shape[1]]
         Normalized root-mean-square deviation between the columns of array and array_ref
     """
 
@@ -180,21 +185,21 @@ def nrmsd(array, array_ref, error_norm="relative", x_axis=False):
 
 def c_map_comparison(c1, c2, t1, t2, nodes, tris):
     """
-    Compares two c-maps in terms of NRMSD and calculates the geodesic distance between the hotspots
+    Compares two c-maps in terms of NRMSD and calculates the geodesic distance between the hotspots.
 
     Parameters
     ----------
-    c1 : ndarray of float [n_ele]
+    c1 : np.ndarray of float [n_ele]
         First c-map
-    c2 : ndarray of float [n_ele]
+    c2 : np.ndarray of float [n_ele]
         Second c-map (reference)
-    t1 : ndarray of float [3]
+    t1 : np.ndarray of float [3]
         Coordinates of the hotspot in the first c-map
-    t2 : ndarray of float [3]
+    t2 : np.ndarray of float [3]
         Coordinates of the hotspot in the second c-map
-    nodes : ndarray of float [n_nodes x 3]
+    nodes : np.ndarray of float [n_nodes x 3]
         Node coordinates
-    tris : ndarray of float [n_tris x 3]
+    tris : np.ndarray of float [n_tris x 3]
         Connectivity of ROI elements
 
     Returns
@@ -204,7 +209,6 @@ def c_map_comparison(c1, c2, t1, t2, nodes, tris):
     gdist : float
         Geodesic distance between the two hotspots in (mm)
     """
-
     # determine NRMSD between two c-maps
     nrmsd_ = nrmsd(array=c1, array_ref=c2, error_norm="relative", x_axis=False) * 100
 
